]> git.sesse.net Git - vlc/blobdiff - src/vlc.c
* OSX lowlevel work
[vlc] / src / vlc.c
index 5f01a83c241a7d0b09e6023daedaf76d0675984a..36cf1f322cd6f97c23901a65a3767805d7273d3b 100644 (file)
--- a/src/vlc.c
+++ b/src/vlc.c
@@ -1,12 +1,13 @@
 /*****************************************************************************
  * vlc.c: the vlc player
  *****************************************************************************
- * Copyright (C) 1998-2001 VideoLAN
- * $Id: vlc.c,v 1.17 2002/11/10 23:41:53 sam Exp $
+ * Copyright (C) 1998-2004 VideoLAN
+ * $Id$
  *
  * Authors: Vincent Seguin <seguin@via.ecp.fr>
  *          Samuel Hocevar <sam@zoy.org>
  *          Gildas Bazin <gbazin@netcourrier.com>
+ *          Derk-Jan Hartman <hartman at videolan dot org>
  *          Lots of other people, see the libvlc AUTHORS file
  *
  * This program is free software; you can redistribute it and/or modify
 #ifdef HAVE_TIME_H
 #   include <time.h>                                               /* time() */
 #endif
-
-#ifdef UNDER_CE
-    /* WinCE needs a WINAPI declaration */
-#   define WIN32_LEAN_AND_MEAN
-#   include <windows.h>
+#ifdef HAVE_STRINGS_H
+#   include <strings.h>                                         /* strncmp() */
 #endif
 
 #include <vlc/vlc.h>
 
+#ifdef SYS_DARWIN
+#include <Cocoa/Cocoa.h>
+#endif
+
 /*****************************************************************************
  * Local prototypes.
  *****************************************************************************/
 static void SigHandler  ( int i_signal );
 #endif
 
+#ifdef SYS_DARWIN
 /*****************************************************************************
- * main: parse command line, start interface and spawn threads
+ * VLCApplication interface
+ *****************************************************************************/
+@interface VLCApplication : NSApplication
+{
+}
+
+@end
+
+/*****************************************************************************
+ * VLCApplication implementation 
+ *****************************************************************************/
+@implementation VLCApplication 
+
+- (void)stop: (id)sender
+{
+    NSEvent *o_event;
+    [super stop:sender];
+
+    /* send a dummy event to break out of the event loop */
+    o_event = [NSEvent mouseEventWithType: NSLeftMouseDown
+                location: NSMakePoint( 1, 1 ) modifierFlags: 0
+                timestamp: 1 windowNumber: [[NSApp mainWindow] windowNumber]
+                context: [NSGraphicsContext currentContext] eventNumber: 1
+                clickCount: 1 pressure: 0.0];
+    [NSApp postEvent: o_event atStart: YES];
+}
+
+- (void)terminate: (id)sender
+{
+    if( [NSApp isRunning] )
+        [NSApp stop:sender];
+    [super terminate: sender];
+}
+
+@end
+
+#endif /* SYS_DARWIN */
+
+/*****************************************************************************
+ * main: parse command line, start interface and spawn threads.
  *****************************************************************************/
-#ifdef UNDER_CE
-int WINAPI WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance,
-                    LPTSTR lpCmdLine, int nCmdShow )
-#else
 int main( int i_argc, char *ppsz_argv[] )
-#endif
 {
     int i_ret;
-#ifdef UNDER_CE
-    int i_argc = 1;
-    char *ppsz_argv[] = { lpCmdLine, NULL };
-#endif
+    int b_cli = VLC_FALSE ;
 
-    fprintf( stderr, "VideoLAN Client %s\n", VLC_Version() );
+#ifndef SYS_DARWIN
+    /* This clutters OSX GUI error logs */
+    fprintf( stderr, "VLC media player %s\n", VLC_Version() );
+#endif
 
 #ifdef HAVE_PUTENV
 #   ifdef DEBUG
@@ -113,18 +150,53 @@ int main( int i_argc, char *ppsz_argv[] )
         return i_ret;
     }
 
-    /* Run libvlc, in non-blocking mode */
-    i_ret = VLC_Play( 0 );
+#ifdef HAVE_STRINGS_H
+    /* if first 3 chars of argv[0] are cli, then this is clivlc
+     * We detect this specifically for Mac OS X, so you can launch vlc
+     * from the commandline even if you are not logged in on the GUI */
+    if( i_argc > 0 )
+    {
+        char *psz_temp;
+        char *psz_program = psz_temp = ppsz_argv[0];
+        while( *psz_temp )
+        {
+            if( *psz_temp == '/' ) psz_program = ++psz_temp;
+            else ++psz_temp;
+        }
+        b_cli = !strncmp( psz_program, "cli", 3 );
+    }
+#endif
+
+#ifdef SYS_DARWIN
+    if( !b_cli )
+    {
+        [VLCApplication sharedApplication];
+    }
 
-    /* Add a blocking interface and keep the return value */
-    i_ret = VLC_AddIntf( 0, NULL, VLC_TRUE );
+    i_ret = VLC_AddIntf( 0, NULL, VLC_TRUE, VLC_TRUE );
+    
+    if( !b_cli )
+    {
+        /* This is a blocking call */
+        [NSApp run];
+    }
+#else
+    i_ret = VLC_AddIntf( 0, NULL, VLC_TRUE, VLC_TRUE );
+#endif /* SYS_DARWIN */
 
     /* Finish the threads */
-    VLC_Stop( 0 );
+    VLC_CleanUp( 0 );
 
     /* Destroy the libvlc structure */
     VLC_Destroy( 0 );
 
+#ifdef SYS_DARWIN
+    if( !b_cli )
+    {
+        [NSApp terminate:NULL];
+    }
+#endif /* SYS_DARWIN */
+
     return i_ret;
 }